#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <signal.h>
#include <errno.h>

#include <ctype.h>
#include <string.h> 
#include <strings.h>

#include <pthread.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include "data.h"
#include "config.h"

#define PORT 79

struct nodedata nodes[MAXNODES];

pthread_t main_thread;
pthread_t data_thread;

int (*update_gui)(void);

char* trimend(char* str)
{
  int s;
  for (s=strlen(str)-1; s>=0 && isspace((int)str[s]); s--)
    str[s] = 0;
  return str;
}

int fingerread(char* buf, int size)
{
  int sock;
  int pos = 0;
  int res; 

  static struct hostent* host;
  struct sockaddr_in serv_addr;


  if (!host) {
    if (!(host = gethostbyname("fabbes.amiga.tm"))) {
      fprintf(stderr,"Error looking up host: %s\n",strerror(h_errno));
      return -1;
    }
  }

  memset((void*)&serv_addr, 0, sizeof(serv_addr));
  serv_addr.sin_family = AF_INET;
  memcpy((void*)&serv_addr.sin_addr.s_addr, (void*)host->h_addr, host->h_length);
  serv_addr.sin_port = htons(PORT);
    
  sock = socket(AF_INET, SOCK_STREAM, 0);
  res = connect(sock, (struct sockaddr*)&serv_addr, sizeof(serv_addr));
  if (res < 0) {
    fprintf(stderr,"Error connecting to server: %s\n",strerror(errno));
    return res;
  }

  res = 1;
  while (res && (pos < size)) {
    res = recv(sock, &buf[pos], size - pos, 0);
    if (res < 0) {
      fprintf(stderr,"Error reading from server: %s\n",strerror(errno));
      return res;
    }
    pos += res;
  }

  close(sock);
  return pos;
}

int update_nodeinfo(void)
{
  static int virgin = 1;

  char fingerbuf[4096];
  char* here;

  struct nodedata* tempdata;

  char user[64];
  char idle[64];
  char action[64];
  char node[64];

  int size;
  int pos;
  int ofs;
  int nnode;

  tempdata = malloc(sizeof(struct nodedata) * MAXNODES);
  memset(tempdata, sizeof(struct nodedata) * MAXNODES, 0);

  if ((size = fingerread(fingerbuf, 4096)) < 0)
    return size;
  
  for (pos = 0; pos < MAXNODES; pos++)
    tempdata[pos].active = 0;

  here = fingerbuf;
  while (here && here[0]) {
    if ((here = strstr(here,"\n"))) {
      here++;
      if (!strncmp(here,"Nod ",strlen("Nod "))) {
	if (sscanf(here,"Nod #%i%*26c%34c",&nnode,(char*)&user) == 2) {
	  if ((user[0] != '<') && (nnode < MAXNODES)) { 
	    // Only interesting in nodes with people
	    
	    // Terminate user string
	    user[34] = 0;

	    // This is where we find idle time (offset from current row)
	    sprintf(node,"%i",nnode);
	    ofs = strlen("Nod #")+strlen(node)+26+34;
	   
	    // Copy it, terminate on null or newline
	    for (pos=0; here[pos+ofs] && (here[pos+ofs] != '\n'); pos++)
	      idle[pos] = here[pos+ofs];
	    idle[pos] = 0;
	   
	    // Go to next line, stop parsing if this is the last
	    if (!(here = strstr(here,"\n")))
	      break;
	    here++;

	    // Check to see that the string isn't terminated
	    for (pos=0; here[pos] && pos < 32; pos++);
	    if (!here[pos])
	      break;
	    
	    // Copy the action, terminate on newline or null
	    for (pos=0; here[pos+32] && (here[pos+32] != '\n'); pos++)
	      action[pos] = here[pos+32];
	    action[pos] = 0;

	    // All parts parsed, save data in struct
	    tempdata[nnode].active = 1;
	    tempdata[nnode].node = strdup(trimend(node));
	    tempdata[nnode].user = strdup(trimend(user));
	    tempdata[nnode].idle = strdup(trimend(idle));
	    tempdata[nnode].action = strdup(trimend(action));
	  }
	}
      }
    }
  }

  // All data for the new code in tempdata, compare with old one
  if (virgin) {
    for (nnode = 0; nnode < MAXNODES; nnode++) {
      if (tempdata[nnode].active) {
	nodes[nnode].active = 1;
	nodes[nnode].node   = tempdata[nnode].node;
	nodes[nnode].user   = tempdata[nnode].user;
	nodes[nnode].action = tempdata[nnode].action;
	nodes[nnode].idle   = tempdata[nnode].idle;
      }
    }
    virgin = 0;
  }
  else {
    for (nnode = 0; nnode < MAXNODES; nnode++) {
      if (tempdata[nnode].active) {
	if (nodes[nnode].active) {
	  nodes[nnode].new = 0;
	  nodes[nnode].changed = 
	    (strcmp(nodes[nnode].user, tempdata[nnode].user) ||
	     strcmp(nodes[nnode].action, tempdata[nnode].action)) ? 1 : 0;
	  
	  free(nodes[nnode].node);
	  free(nodes[nnode].user);
	  free(nodes[nnode].action);
	  free(nodes[nnode].idle);
	}
	else {
	  nodes[nnode].active = 1;
	  nodes[nnode].new = 1;
	  nodes[nnode].changed = 1;
	  nodes[nnode].monitor = 0;
	}
	nodes[nnode].node   = tempdata[nnode].node;
	nodes[nnode].user   = tempdata[nnode].user;
	nodes[nnode].action = tempdata[nnode].action;
	nodes[nnode].idle   = tempdata[nnode].idle;
      }
      else {
	if (nodes[nnode].active) {
	  free(nodes[nnode].node);
	  free(nodes[nnode].user);
	  free(nodes[nnode].action);
	  free(nodes[nnode].idle);
	}
	nodes[nnode].active = 0;
      }
    }
  }
  return 0;
}

void* update_loop(void* dummy)
{
  while (1) {
    if (!update_nodeinfo())
      pthread_kill(main_thread, SIGUSR1);
    sleep(config.update_delay);
  }
}

void update(int dummy)
{
  update_gui();
  signal(SIGUSR1,update);
}

int data_init(int argc, char* argv[], int (*update_func)(void))
{
  update_gui = update_func;
  signal(SIGUSR1, update);
  main_thread = pthread_self();
  pthread_create(&data_thread, NULL, update_loop, NULL);
  return 0;
}

