#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include <stdio.h>
#include <stdlib.h>

#include "data.h"
#include "gui.h"
#include "config.h"

GtkWidget* window;
GtkWidget* scroll;
GtkWidget* userlist;

enum sort_t {NodeA, NodeD, NameA, NameD, ActionA, ActionD, IdleA, IdleD};
enum sort_t sort;

void destroy(GtkWidget *widget, gpointer data)
{
  gtk_main_quit();
}

gboolean monitor_node(GtkWidget* widget, GdkEventButton* event, gpointer user_data)
{
  gint ret;
  gint row,col;
  int nnode;
  char* node;

  if (!event)
    return TRUE;

  if (event->button==1) {
    ret = gtk_clist_get_selection_info(GTK_CLIST(widget), 
				       (gint) event->x, 
				       (gint) event->y, &row, &col);

    if (ret && (row < GTK_CLIST(widget)->rows) && (event->type == GDK_2BUTTON_PRESS)) {
      if (gtk_clist_get_text(GTK_CLIST(widget),row,0,&node)) {
	nnode = atoi(node);
	nodes[nnode].monitor = !nodes[nnode].monitor;
	update_userlist();
	return TRUE;
      }
    }
  }
  return FALSE;
}

void change_sort(GtkCList* clist, gint column, gpointer user_data)
{
  switch (column) {
    case 0: sort = ((sort != NodeA)  ? NodeA   : NodeD); break;
    case 1: sort = (sort != NameA)   ? NameA   : NameD; break;
    case 2: sort = (sort != ActionA) ? ActionA : ActionD; break;
    case 3: sort = (sort != IdleA)   ? IdleA   : IdleD; break;
  }
  update_userlist();
}

void confToGdkColor(GdkColor* dcol, color clr)
{
  dcol->pixel = 0;
  dcol->red   = (clr >> 16 & 0xff) << 8;
  dcol->green = (clr >> 8 & 0xff) << 8;
  dcol->blue  = (clr & 0xff) << 8;
}

gint list_comp_function(GtkCList* clist, gconstpointer ptr1, gconstpointer ptr2)
{
  int col;

  char* str1;
  char* str2;

  GtkCListRow* row1 = (GtkCListRow*)ptr1;
  GtkCListRow* row2 = (GtkCListRow*)ptr2;

  switch (sort) {
  case NodeA:
  case NodeD: col = 0; break;
  case NameA:
  case NameD: col = 1; break;
  case ActionA:
  case ActionD: col = 2; break;
  case IdleA:
  case IdleD: col = 3; break;
  }

  if ((row1->cell[col].type != GTK_CELL_TEXT) || 
      (row2->cell[col].type != GTK_CELL_TEXT))
    return 0;

  str1 = GTK_CELL_TEXT(row1->cell[col])->text;
  str2 = GTK_CELL_TEXT(row2->cell[col])->text;

  switch (sort) {
  case NodeA:
  case NameA:
  case ActionA: 
  case IdleA: return strcmp(str1,str2);
  case NodeD:
  case NameD:
  case ActionD:
  case IdleD: return -strcmp(str1,str2);
  }
  return 0;
}


int update_userlist(void)
{
  int s;
  int rown;
  int users = 0;

  static gchar* row[4];
  static gchar* nousers[] = {"","Inga inloggade","",""};
  static gchar buf[64];

  GdkColor newClr;
  GdkColor chngClr;
  GdkColor highClr;
  GdkColor monClr;

  confToGdkColor(&newClr,  config.color_new);
  confToGdkColor(&chngClr, config.color_change);
  confToGdkColor(&highClr, config.color_highlight);
  confToGdkColor(&monClr,  config.color_monitored);
  
  gtk_clist_clear(GTK_CLIST(userlist));

  for (s=0; s < MAXNODES; s++) {
    if (nodes[s].active) {
      row[0] = nodes[s].node;
      row[1] = nodes[s].user;
      row[2] = nodes[s].action;
      row[3] = nodes[s].idle;
      rown = gtk_clist_append(GTK_CLIST(userlist), row);
      gtk_clist_set_selectable(GTK_CLIST(userlist), rown, FALSE);

      if (nodes[s].changed) {
	if (nodes[s].monitor) {
	  nodes[s].monitor = 0;
	  gdk_beep();
	  gtk_clist_set_background(GTK_CLIST(userlist), rown, &highClr);
	}
	else
	  gtk_clist_set_background(GTK_CLIST(userlist), rown, &chngClr);
      }
      else if (nodes[s].monitor)
	gtk_clist_set_background(GTK_CLIST(userlist), rown, &monClr);
      else if (nodes[s].new)
	gtk_clist_set_background(GTK_CLIST(userlist), rown, &newClr);

      users++;
    }
  }
  
  if (users)
    gtk_clist_sort(GTK_CLIST(userlist));
  else
    gtk_clist_append(GTK_CLIST(userlist), nousers);
  
  sprintf(buf,"FabbMon - %i anvndare inloggade",users);
  gtk_window_set_title(GTK_WINDOW(window), buf);

  return 0;
}

int gui_init(int argc, char* argv[])
{
  gchar* titles[] = {"Nod","Anvndare","Gr","Idle"};
  gchar* reading_data[] = {"","Hmtar data","",""};
  gtk_init(&argc, &argv);

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect (GTK_OBJECT(window), "destroy",GTK_SIGNAL_FUNC (destroy), NULL);
  gtk_window_set_default_size(GTK_WINDOW(window),384,84);

  scroll = gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), 
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  userlist = gtk_clist_new_with_titles(4,titles);
  gtk_signal_connect (GTK_OBJECT(userlist), 
		      "button_press_event",GTK_SIGNAL_FUNC(monitor_node), NULL);
  gtk_signal_connect (GTK_OBJECT(userlist), 
		      "click-column",GTK_SIGNAL_FUNC(change_sort), NULL);

  gtk_clist_set_compare_func(GTK_CLIST(userlist), list_comp_function);
  gtk_clist_sort(GTK_CLIST(userlist));

  gtk_clist_set_column_width(GTK_CLIST(userlist),0,20);
  gtk_clist_set_column_width(GTK_CLIST(userlist),1,150);
  gtk_clist_set_column_width(GTK_CLIST(userlist),2,150);
  gtk_clist_set_column_width(GTK_CLIST(userlist),3,30);
  gtk_clist_set_column_visibility(GTK_CLIST(userlist), 0, FALSE);

  gtk_clist_append(GTK_CLIST(userlist), reading_data);
  gtk_window_set_title(GTK_WINDOW(window), "FabbMon - hmtar data");

  gtk_container_add(GTK_CONTAINER(window), scroll);
  gtk_container_add(GTK_CONTAINER(scroll), userlist);

  gtk_widget_show(userlist);
  gtk_widget_show(scroll);
  gtk_widget_show(window);

  sort = Node;
  return 0;
}

int gui_loop()
{
  gtk_main();
  return 0;
}
